from enum import Enum
from toloka.client.project.template_builder.base import (
    BaseTemplate,
    VersionedBaseComponent,
    BaseComponent
)
from typing import (
    List,
    Any,
    Dict,
    Union,
    Optional
)


class BasePluginV1(VersionedBaseComponent):
    """Plugins that provide expanded functionality. For example, you can use plugin.hotkeys to set up shortcuts.
    """

    def __init__(self, *, version: Optional[str] = '1.0.0') -> None:
        """Method generated by attrs for class BasePluginV1.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    version: Optional[str]

class HotkeysPluginV1(BasePluginV1):
    """Lets you set keyboard shortcuts for actions.

    Attributes:
        key_ + [a-z|0-9|up|down]: An action that is triggered when you press the specified keyboard key. The keyboard
            shortcut is set in the key, and the action is specified in the value
    """

    def __init__(self, *, version: Optional[str] = '1.0.0', key_a: Optional[Any] = None, key_b: Optional[Any] = None, key_c: Optional[Any] = None, key_d: Optional[Any] = None, key_e: Optional[Any] = None, key_f: Optional[Any] = None, key_g: Optional[Any] = None, key_h: Optional[Any] = None, key_i: Optional[Any] = None, key_j: Optional[Any] = None, key_k: Optional[Any] = None, key_l: Optional[Any] = None, key_m: Optional[Any] = None, key_n: Optional[Any] = None, key_o: Optional[Any] = None, key_p: Optional[Any] = None, key_q: Optional[Any] = None, key_r: Optional[Any] = None, key_s: Optional[Any] = None, key_t: Optional[Any] = None, key_u: Optional[Any] = None, key_v: Optional[Any] = None, key_w: Optional[Any] = None, key_x: Optional[Any] = None, key_y: Optional[Any] = None, key_z: Optional[Any] = None, key_0: Optional[Any] = None, key_1: Optional[Any] = None, key_2: Optional[Any] = None, key_3: Optional[Any] = None, key_4: Optional[Any] = None, key_5: Optional[Any] = None, key_6: Optional[Any] = None, key_7: Optional[Any] = None, key_8: Optional[Any] = None, key_9: Optional[Any] = None, key_up: Optional[Any] = None, key_down: Optional[Any] = None) -> None:
        """Method generated by attrs for class HotkeysPluginV1.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    version: Optional[str]
    key_a: Optional[Any]
    key_b: Optional[Any]
    key_c: Optional[Any]
    key_d: Optional[Any]
    key_e: Optional[Any]
    key_f: Optional[Any]
    key_g: Optional[Any]
    key_h: Optional[Any]
    key_i: Optional[Any]
    key_j: Optional[Any]
    key_k: Optional[Any]
    key_l: Optional[Any]
    key_m: Optional[Any]
    key_n: Optional[Any]
    key_o: Optional[Any]
    key_p: Optional[Any]
    key_q: Optional[Any]
    key_r: Optional[Any]
    key_s: Optional[Any]
    key_t: Optional[Any]
    key_u: Optional[Any]
    key_v: Optional[Any]
    key_w: Optional[Any]
    key_x: Optional[Any]
    key_y: Optional[Any]
    key_z: Optional[Any]
    key_0: Optional[Any]
    key_1: Optional[Any]
    key_2: Optional[Any]
    key_3: Optional[Any]
    key_4: Optional[Any]
    key_5: Optional[Any]
    key_6: Optional[Any]
    key_7: Optional[Any]
    key_8: Optional[Any]
    key_9: Optional[Any]
    key_up: Optional[Any]
    key_down: Optional[Any]

class TriggerPluginV1(BasePluginV1):
    """Use this to configure triggers that trigger a specific action when an event occurs.

    The action is set in the action property, and the event is described in the other fields.

    The event can be triggered immediately when the task is loaded ("fireImmediately": true) or when data changes in
    the property specified in onChangeOf.

    You can also set conditions in the conditions property that must be met in order for the trigger to fire.
    Attributes:
        action: The action to perform when the trigger fires.
        condition: The condition that must be met in order to fire the trigger.
        fire_immediately: Flag indicating whether the trigger should be fired immediately after the task is loaded.
        on_change_of: The data that triggers the action when changed.

    Example:
        How to save the performer coordinates to the output.

        >>> coordinates_save_plugin = tb.plugins.TriggerPluginV1(
        >>>     fire_immediately=True,
        >>>     action=tb.actions.SetActionV1(
        >>>         data=tb.data.OutputData(path='performer_coordinates'),
        >>>         payload=tb.data.LocationData()
        >>>     ),
        >>> )
        ...
    """

    def __init__(self, *, version: Optional[str] = '1.0.0', action: Optional[BaseComponent] = None, condition: Optional[BaseComponent] = None, fire_immediately: Optional[Union[BaseComponent, bool]] = None, on_change_of: Optional[BaseComponent] = None) -> None:
        """Method generated by attrs for class TriggerPluginV1.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    version: Optional[str]
    action: Optional[BaseComponent]
    condition: Optional[BaseComponent]
    fire_immediately: Optional[Union[BaseComponent, bool]]
    on_change_of: Optional[BaseComponent]

class TolokaPluginV1(BasePluginV1):
    """A plugin with extra settings for tasks in Toloka.

    Attributes:
        layout: Settings for the task appearance in Toloka.
        notifications: Notifications shown at the top of the page.

    Example:
        How to set the task width on the task page.

        >>> task_width_plugin = tb.plugins.TolokaPluginV1(
        >>>     layout = tb.plugins.TolokaPluginV1.TolokaPluginLayout(
        >>>         kind='scroll',
        >>>         task_width=400,
        >>>     )
        >>> )
        ...
    """

    class TolokaPluginLayout(BaseTemplate):
        """How to display task.
        """

        class Kind(Enum):
            """scroll (default) — display multiple tasks on the page at the same time.

            pager — display only one task on the page, with a button to switch between tasks at the bottom.
            """

            PAGER = 'pager'
            SCROLL = 'scroll'



        def __init__(self, *, kind: Optional[Kind] = ..., task_width: Optional[float] = None) -> None:
            """Method generated by attrs for class TolokaPluginV1.TolokaPluginLayout.
            """
            ...

        _unexpected: Optional[Dict[str, Any]]
        kind: Optional[Kind]
        task_width: Optional[float]



    def __init__(self, *, version: Optional[str] = '1.0.0', layout: Optional[Union[BaseComponent, TolokaPluginLayout]] = ..., notifications: Optional[Union[BaseComponent, List[BaseComponent]]] = None) -> None:
        """Method generated by attrs for class TolokaPluginV1.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    version: Optional[str]
    layout: Optional[Union[BaseComponent, TolokaPluginLayout]]
    notifications: Optional[Union[BaseComponent, List[BaseComponent]]]

